@@ -20,7 +20,10 @@
 #
 # Last changed: 2010-01-28
 
-from . import finite_element, polynomial_set, dual_set, functional
+from __future__ import absolute_import, print_function, division
+
+from FIAT import finite_element, polynomial_set, dual_set, functional
+
 
 def _initialize_entity_ids(topology):
     entity_ids = {}
@@ -30,6 +33,7 @@ def _initialize_entity_ids(topology):
             entity_ids[i][j] = []
     return entity_ids
 
+
 class CrouzeixRaviartDualSet(dual_set.DualSet):
     """Dual basis for Crouzeix-Raviart element (linears continuous at
     boundary midpoints)."""
@@ -42,22 +46,23 @@ class CrouzeixRaviartDualSet(dual_set.DualSet):
 
         # Initialize empty nodes and entity_ids
         entity_ids = _initialize_entity_ids(topology)
-        nodes = [None for i in list(topology[d-1].keys())]
+        nodes = [None for i in list(topology[d - 1].keys())]
 
         # Construct nodes and entity_ids
-        for i in topology[d-1]:
+        for i in topology[d - 1]:
 
             # Construct midpoint
-            x = cell.make_points(d-1, i, d)[0]
+            x = cell.make_points(d - 1, i, d)[0]
 
             # Degree of freedom number i is evaluation at midpoint
             nodes[i] = functional.PointEvaluation(cell, x)
-            entity_ids[d-1][i] += [i]
+            entity_ids[d - 1][i] += [i]
 
         # Initialize super-class
-        dual_set.DualSet.__init__(self, nodes, cell, entity_ids)
+        super(CrouzeixRaviartDualSet, self).__init__(nodes, cell, entity_ids)
 
-class CrouzeixRaviart(finite_element.FiniteElement):
+
+class CrouzeixRaviart(finite_element.CiarletElement):
     """The Crouzeix-Raviart finite element:
 
     K:                 Triangle/Tetrahedron
@@ -75,17 +80,4 @@ class CrouzeixRaviart(finite_element.FiniteElement):
         # FiniteElement
         space = polynomial_set.ONPolynomialSet(cell, 1)
         dual = CrouzeixRaviartDualSet(cell, 1)
-        finite_element.FiniteElement.__init__(self, space, dual, 1)
-
-
-if __name__ == "__main__":
-
-    from . import reference_element
-
-    cells = [reference_element.UFCTriangle(),
-             reference_element.UFCTetrahedron()]
-    for cell in cells:
-        print("Checking CrouzeixRaviart(cell, 1)")
-        element = CrouzeixRaviart(cell, 1)
-        print([L.pt_dict for L in element.dual_basis()])
-        print()
+        super(CrouzeixRaviart, self).__init__(space, dual, 1)
