@@ -29,7 +29,7 @@ from FIAT.lagrange import Lagrange
 from FIAT.discontinuous_lagrange import DiscontinuousLagrange
 from FIAT.nedelec import Nedelec
 from FIAT.raviart_thomas import RaviartThomas
-from FIAT.tensor_product import TensorProductElement
+from FIAT.tensor_product import TensorProductElement, FlattenedDimensions
 from FIAT.hdivcurl import Hdiv, Hcurl
 from FIAT.enriched import EnrichedElement
 
@@ -522,6 +522,46 @@ def test_TFE_2Dx1D_vector_quad_hcurl():
         assert tab[dd][7][2][0] == 0.0
 
 
+def test_flattened_against_tpe_quad():
+    T = UFCInterval()
+    P1 = Lagrange(T, 1)
+    tpe_quad = TensorProductElement(P1, P1)
+    flattened_quad = FlattenedDimensions(tpe_quad)
+    assert tpe_quad.value_shape() == ()
+    tpe_tab = tpe_quad.tabulate(1, [(0.1, 0.2)])
+    flattened_tab = flattened_quad.tabulate(1, [(0.1, 0.2)])
+
+    for da, db in [[(0,), (0,)], [(1,), (0,)], [(0,), (1,)]]:
+        dc = da + db
+        assert np.isclose(tpe_tab[dc][0][0], flattened_tab[dc][0][0])
+        assert np.isclose(tpe_tab[dc][1][0], flattened_tab[dc][1][0])
+        assert np.isclose(tpe_tab[dc][2][0], flattened_tab[dc][2][0])
+        assert np.isclose(tpe_tab[dc][3][0], flattened_tab[dc][3][0])
+
+
+def test_flattened_against_tpe_hex():
+    T = UFCInterval()
+    P1 = Lagrange(T, 1)
+    tpe_quad = TensorProductElement(P1, P1)
+    tpe_hex = TensorProductElement(tpe_quad, P1)
+    flattened_quad = FlattenedDimensions(tpe_quad)
+    flattened_hex = FlattenedDimensions(TensorProductElement(flattened_quad, P1))
+    assert tpe_quad.value_shape() == ()
+    tpe_tab = tpe_hex.tabulate(1, [(0.1, 0.2, 0.3)])
+    flattened_tab = flattened_hex.tabulate(1, [(0.1, 0.2, 0.3)])
+
+    for da, db, dc in [[(0,), (0,), (0,)], [(1,), (0,), (0,)], [(0,), (1,), (0,)], [(0,), (0,), (1,)]]:
+        dd = da + db + dc
+        assert np.isclose(tpe_tab[dd][0][0], flattened_tab[dd][0][0])
+        assert np.isclose(tpe_tab[dd][1][0], flattened_tab[dd][1][0])
+        assert np.isclose(tpe_tab[dd][2][0], flattened_tab[dd][2][0])
+        assert np.isclose(tpe_tab[dd][3][0], flattened_tab[dd][3][0])
+        assert np.isclose(tpe_tab[dd][4][0], flattened_tab[dd][4][0])
+        assert np.isclose(tpe_tab[dd][5][0], flattened_tab[dd][5][0])
+        assert np.isclose(tpe_tab[dd][6][0], flattened_tab[dd][6][0])
+        assert np.isclose(tpe_tab[dd][7][0], flattened_tab[dd][7][0])
+
+
 if __name__ == '__main__':
     import os
     pytest.main(os.path.abspath(__file__))
